@@ -39,6 +39,7 @@ from sqlalchemy.types import VARBINARY
 from alembic import autogenerate
 from alembic import testing
 from alembic.autogenerate import api
+from alembic.autogenerate.compare.tables import _compare_tables
 from alembic.migration import MigrationContext
 from alembic.operations import ops
 from alembic.testing import assert_raises_message
@@ -55,8 +56,14 @@ from alembic.testing.suite._autogen_fixtures import _default_name_filters
 from alembic.testing.suite._autogen_fixtures import _default_object_filters
 from alembic.testing.suite._autogen_fixtures import AutogenFixtureTest
 from alembic.testing.suite._autogen_fixtures import AutogenTest
+from alembic.testing.suite._autogen_fixtures import ModelOne
 from alembic.util import CommandError
 
+if True:
+    from alembic.autogenerate.compare.types import (
+        _dialect_impl_compare_type as _compare_type,
+    )
+
 # TODO: we should make an adaptation of CompareMetadataToInspectorTest that is
 #       more well suited towards generic backends (2021-06-10)
 
@@ -237,7 +244,7 @@ class AutogenDefaultSchemaTest(AutogenFixtureTest, TestBase):
     __only_on__ = "postgresql"
     __backend__ = True
 
-    def test_uses_explcit_schema_in_default_one(self):
+    def test_uses_explicit_schema_in_default_one(self):
         default_schema = self.bind.dialect.default_schema_name
 
         m1 = MetaData()
@@ -249,7 +256,7 @@ class AutogenDefaultSchemaTest(AutogenFixtureTest, TestBase):
         diffs = self._fixture(m1, m2, include_schemas=True)
         eq_(diffs, [])
 
-    def test_uses_explcit_schema_in_default_two(self):
+    def test_uses_explicit_schema_in_default_two(self):
         default_schema = self.bind.dialect.default_schema_name
 
         m1 = MetaData()
@@ -265,7 +272,7 @@ class AutogenDefaultSchemaTest(AutogenFixtureTest, TestBase):
         eq_(diffs[0][1].schema, "test_schema")
         eq_(diffs[0][1].c.keys(), ["y"])
 
-    def test_uses_explcit_schema_in_default_three(self):
+    def test_uses_explicit_schema_in_default_three(self):
         default_schema = self.bind.dialect.default_schema_name
 
         m1 = MetaData()
@@ -315,104 +322,6 @@ class AutogenDefaultSchemaIsNoneTest(AutogenFixtureTest, TestBase):
         eq_(len(diffs), 0)
 
 
-class ModelOne:
-    __requires__ = ("unique_constraint_reflection",)
-
-    schema = None
-
-    @classmethod
-    def _get_db_schema(cls):
-        schema = cls.schema
-
-        m = MetaData(schema=schema)
-
-        Table(
-            "user",
-            m,
-            Column("id", Integer, primary_key=True),
-            Column("name", String(50)),
-            Column("a1", Text),
-            Column("pw", String(50)),
-            Index("pw_idx", "pw"),
-        )
-
-        Table(
-            "address",
-            m,
-            Column("id", Integer, primary_key=True),
-            Column("email_address", String(100), nullable=False),
-        )
-
-        Table(
-            "order",
-            m,
-            Column("order_id", Integer, primary_key=True),
-            Column(
-                "amount",
-                Numeric(8, 2),
-                nullable=False,
-                server_default=text("0"),
-            ),
-            CheckConstraint("amount >= 0", name="ck_order_amount"),
-        )
-
-        Table(
-            "extra",
-            m,
-            Column("x", CHAR),
-            Column("uid", Integer, ForeignKey("user.id")),
-        )
-
-        return m
-
-    @classmethod
-    def _get_model_schema(cls):
-        schema = cls.schema
-
-        m = MetaData(schema=schema)
-
-        Table(
-            "user",
-            m,
-            Column("id", Integer, primary_key=True),
-            Column("name", String(50), nullable=False),
-            Column("a1", Text, server_default="x"),
-        )
-
-        Table(
-            "address",
-            m,
-            Column("id", Integer, primary_key=True),
-            Column("email_address", String(100), nullable=False),
-            Column("street", String(50)),
-            UniqueConstraint("email_address", name="uq_email"),
-        )
-
-        Table(
-            "order",
-            m,
-            Column("order_id", Integer, primary_key=True),
-            Column(
-                "amount",
-                Numeric(10, 2),
-                nullable=True,
-                server_default=text("0"),
-            ),
-            Column("user_id", Integer, ForeignKey("user.id")),
-            CheckConstraint("amount > -1", name="ck_order_amount"),
-        )
-
-        Table(
-            "item",
-            m,
-            Column("id", Integer, primary_key=True),
-            Column("description", String(100)),
-            Column("order_id", Integer, ForeignKey("order.order_id")),
-            CheckConstraint("len(description) > 5"),
-        )
-        return m
-
-
 class AutogenerateDiffTest(ModelOne, AutogenTest, TestBase):
     __only_on__ = "sqlite"
 
@@ -597,7 +506,7 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestBase):
 
     def test_skip_null_type_comparison_reflected(self):
         ac = ops.AlterColumnOp("sometable", "somecol")
-        autogenerate.compare._compare_type(
+        _compare_type(
             self.autogen_context,
             ac,
             None,
@@ -611,7 +520,7 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestBase):
 
     def test_skip_null_type_comparison_local(self):
         ac = ops.AlterColumnOp("sometable", "somecol")
-        autogenerate.compare._compare_type(
+        _compare_type(
             self.autogen_context,
             ac,
             None,
@@ -627,11 +536,8 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestBase):
         class MyType(TypeDecorator):
             impl = Integer
 
-            def compare_against_backend(self, dialect, conn_type):
-                return isinstance(conn_type, Integer)
-
         ac = ops.AlterColumnOp("sometable", "somecol")
-        autogenerate.compare._compare_type(
+        _compare_type(
             self.autogen_context,
             ac,
             None,
@@ -644,7 +550,7 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestBase):
         assert not ac.has_changes()
 
         ac = ops.AlterColumnOp("sometable", "somecol")
-        autogenerate.compare._compare_type(
+        _compare_type(
             self.autogen_context,
             ac,
             None,
@@ -667,7 +573,7 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestBase):
                     return dialect.type_descriptor(CHAR(32))
 
         uo = ops.AlterColumnOp("sometable", "somecol")
-        autogenerate.compare._compare_type(
+        _compare_type(
             self.autogen_context,
             uo,
             None,
@@ -683,7 +589,7 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestBase):
 
         inspector = inspect(self.bind)
         uo = ops.UpgradeOps(ops=[])
-        autogenerate.compare._compare_tables(
+        _compare_tables(
             OrderedSet([(None, "extra"), (None, "user")]),
             OrderedSet(),
             inspector,
@@ -699,6 +605,17 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestBase):
             ],
         )
 
+    def test_add_execute_sql_op(self):
+        uo = ops.UpgradeOps(ops=[])
+        autogenerate._produce_net_changes(self.autogen_context, uo)
+
+        uo.ops.append(ops.ExecuteSQLOp("STATEMENT"))
+
+        diffs = uo.as_diffs()
+
+        eq_(diffs[-1][0], "execute")
+        eq_(diffs[-1][1], "STATEMENT")
+
 
 class AutogenerateDiffTestWSchema(ModelOne, AutogenTest, TestBase):
     __only_on__ = "postgresql"
@@ -967,6 +884,63 @@ class CompareServerDefaultTest(TestBase):
 
         assert not diff
 
+    @testing.combinations(
+        (False, "different", False),
+        (True, "original", True),
+        (None, "different", True),
+        id_="naa",
+        argnames="return_value,new_default,expect_diff",
+    )
+    def test_user_compare_server_default_return_values(
+        self, return_value, new_default, expect_diff, connection, metadata
+    ):
+        """Test user compare_server_default callable return values.
+
+        This is a regression test for #1777 where the plugin refactoring
+        broke the handling of False return values from user-defined
+        compare_server_default callables.
+
+        - False: stop comparison, no diff detected
+        - True: diff detected regardless of actual difference
+        - None: continue to default comparison logic
+        """
+
+        def my_compare_server_default(
+            context,
+            inspected_col,
+            metadata_col,
+            inspected_default,
+            metadata_default,
+            rendered_metadata_default,
+        ):
+            return return_value
+
+        t1 = Table(
+            "t1",
+            metadata,
+            Column("x", VARCHAR(30), server_default=text("'original'")),
+        )
+        t1.create(connection)
+
+        new_metadata = MetaData()
+        Table(
+            "t1",
+            new_metadata,
+            Column("x", VARCHAR(30), server_default=text(f"'{new_default}'")),
+        )
+
+        mc = MigrationContext.configure(
+            connection,
+            opts={"compare_server_default": my_compare_server_default},
+        )
+
+        diff = api.compare_metadata(mc, new_metadata)
+        if expect_diff:
+            eq_(len(diff), 1)
+            eq_(diff[0][0][0], "modify_default")
+        else:
+            assert not diff
+
 
 class CompareMetadataToInspectorTest(TestBase):
     __backend__ = True
@@ -1293,12 +1267,12 @@ class AutogenerateCustomCompareTypeTest(AutogenTest, TestBase):
         my_compare_type.return_value = False
         self.context._user_compare_type = my_compare_type
 
-        diffs = []
         ctx = self.autogen_context
-        diffs = []
-        autogenerate._produce_net_changes(ctx, diffs)
 
-        eq_(diffs, [])
+        uo = ops.UpgradeOps(ops=[])
+        autogenerate._produce_net_changes(ctx, uo)
+
+        eq_(uo.as_diffs(), [])
 
     def test_column_type_modified_custom_compare_type_returns_True(self):
         my_compare_type = mock.Mock()
@@ -1775,6 +1749,7 @@ class OrigObjectTest(TestBase):
             Column("x", Integer()),
         )
         self.ix = Index("ix1", t.c.id)
+        self.ix_unique = Index("ix2", t.c.id, unique=True)
         fk = ForeignKeyConstraint(["t_id"], ["t.id"])
         q = Table("q", m, Column("t_id", Integer()), fk)
         self.table = t
@@ -1897,6 +1872,14 @@ class OrigObjectTest(TestBase):
         eq_(op.to_index(), schemacompare.CompareIndex(self.ix))
         eq_(op.reverse().to_index(), schemacompare.CompareIndex(self.ix))
 
+    def test_create_unique_index(self):
+        op = ops.CreateIndexOp.from_index(self.ix_unique)
+        eq_(op.to_index(), schemacompare.CompareIndex(self.ix_unique))
+        eq_(
+            op.reverse().to_index(),
+            schemacompare.CompareIndex(self.ix_unique),
+        )
+
 
 class MultipleMetaDataTest(AutogenFixtureTest, TestBase):
     def test_multiple(self):
@@ -1990,3 +1973,249 @@ class MultipleMetaDataTest(AutogenFixtureTest, TestBase):
             [m1a, m1b],
             [m2a, m2b],
         )
+
+
+class AutogenFKTest(AutogenFixtureTest, TestBase):
+    """Test foreign key constraint handling in autogenerate."""
+
+    __backend__ = True
+    __requires__ = ("foreign_key_constraint_reflection",)
+
+    @testing.variation("case", ["uppercase", "lowercase"])
+    @testing.variation("use_naming_convention", [True, False])
+    @config.requirements.foreign_key_name_reflection
+    def test_drop_fk_with_mixed_case_name(self, case, use_naming_convention):
+        """Test #1743"""
+        if case.uppercase:
+            fk_pattern = (
+                "FK_%(table_name)s_%(column_0_name)s_%(referred_table_name)s"
+            )
+            expected_name = "FK_child_parent_id_parent"
+        else:
+            fk_pattern = (
+                "fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s"
+            )
+            expected_name = "fk_child_parent_id_parent"
+
+        if use_naming_convention:
+            m1 = MetaData(naming_convention={"fk": fk_pattern})
+        else:
+            m1 = MetaData()
+
+        Table(
+            "parent",
+            m1,
+            Column("id", Integer, primary_key=True),
+            Column("name", String(50)),
+        )
+
+        c = Table(
+            "child",
+            m1,
+            Column("id", Integer, primary_key=True),
+            Column("name", String(50)),
+            Column("parent_id", Integer),
+            (
+                ForeignKeyConstraint(["parent_id"], ["parent.id"])
+                if use_naming_convention
+                else ForeignKeyConstraint(
+                    ["parent_id"], ["parent.id"], name=expected_name
+                )
+            ),
+        )
+
+        eq_(
+            [
+                fk
+                for fk in c.constraints
+                if isinstance(fk, ForeignKeyConstraint)
+            ][0].name,
+            expected_name,
+        )
+
+        if use_naming_convention:
+            m2 = MetaData(naming_convention={"fk": fk_pattern})
+        else:
+            m2 = MetaData()
+
+        Table(
+            "parent",
+            m2,
+            Column("id", Integer, primary_key=True),
+            Column("name", String(50)),
+        )
+
+        Table(
+            "child",
+            m2,
+            Column("id", Integer, primary_key=True),
+            Column("name", String(50)),
+            Column("parent_id", Integer),
+        )
+
+        diffs = self._fixture(m1, m2)
+
+        # Should detect the FK removal
+        eq_(len(diffs), 1, f"Expected 1 diff, got {len(diffs)}: {diffs}")
+
+        # Verify it's a remove_fk operation with the correct name
+        self._assert_fk_diff(
+            diffs[0],
+            "remove_fk",
+            "child",
+            ["parent_id"],
+            "parent",
+            ["id"],
+            name=expected_name,
+        )
+
+
+class AutogenInspectorCacheTest(AutogenFixtureTest, TestBase):
+    """test for the new inspector caching added for #1771."""
+
+    __only_on__ = ("sqlite", "postgresql", "oracle")
+    __requires__ = ("sqlalchemy_2",)
+
+    @testing.fixture
+    def instrument_inspector_conv(self, connection):
+        from sqlalchemy import event
+
+        shared_mock = mock.MagicMock()
+
+        def track_before_cursor_execute(
+            conn, cursor, statement, parameters, context, executemany
+        ):
+            shared_mock.sql_call(statement)
+
+        event.listen(
+            connection,
+            "before_cursor_execute",
+            track_before_cursor_execute,
+        )
+
+        yield shared_mock
+
+        event.remove(
+            connection,
+            "before_cursor_execute",
+            track_before_cursor_execute,
+        )
+
+    @testing.fixture
+    def models(self, metadata, connection):
+        m1 = metadata
+        m2 = MetaData()
+        from sqlalchemy import Identity
+
+        Table(
+            "parent",
+            m1,
+            Column("id", Integer, Identity(), primary_key=True),
+            Column("name", String(50)),
+            Column("x", String(50)),
+            Column("y", String(50)),
+        )
+
+        Table(
+            "child",
+            m1,
+            Column("id", Integer, Identity(), primary_key=True),
+            Column("name", String(50)),
+            Column("parent_id", Integer),
+            (ForeignKeyConstraint(["parent_id"], ["parent.id"])),
+        )
+
+        Table(
+            "t1",
+            m1,
+            Column("id", Integer, Identity(), primary_key=True),
+            Column("name", String(50)),
+        )
+
+        Table(
+            "t2",
+            m1,
+            Column("id", Integer, Identity(), primary_key=True),
+            Column("name", String(50)),
+        )
+
+        Table(
+            "t3",
+            m1,
+            Column("id", Integer, Identity(), primary_key=True),
+            Column("name", String(50)),
+        )
+
+        m1.create_all(connection)
+
+        ctx_opts = {
+            "compare_type": True,
+            "compare_server_default": True,
+            "target_metadata": m2,
+            "upgrade_token": "upgrades",
+            "downgrade_token": "downgrades",
+            "alembic_module_prefix": "op.",
+            "sqlalchemy_module_prefix": "sa.",
+        }
+        context = MigrationContext.configure(
+            connection=connection, opts=ctx_opts
+        )
+
+        autogen_context = api.AutogenContext(context, m2)
+        uo = ops.UpgradeOps(ops=[])
+
+        yield autogen_context, uo
+
+    @testing.fixture(params=[True, False])
+    def disable_pre_cache(self, request):
+        from alembic.autogenerate.compare.util import _SQLA2InspectorConv
+
+        patcher = mock.patch.object(_SQLA2InspectorConv, "pre_cache_tables")
+
+        param = request.param
+
+        if param:
+            patcher.start()
+
+        yield param
+
+        if param:
+            patcher.stop()
+
+    class Approx:
+        def __init__(self, value):
+            self.value = value
+
+        def __eq__(self, other):
+            return abs(other - self.value) < 5
+
+        def __repr__(self):
+            return f"Approximately({self.value})"
+
+    expected = {
+        # not any savings for SQLite which does query-per-table no matter what.
+        # but we can at least see that pre-caching does all the SQL up front
+        "sqlite": {
+            "pre_cache": Approx(60),
+            "no_pre_cache": Approx(95),
+        },
+        # for PG and Oracle which use multi-queries, big savings
+        "postgresql": {"pre_cache": Approx(12), "no_pre_cache": Approx(57)},
+        "oracle": {"pre_cache": Approx(12), "no_pre_cache": Approx(32)},
+    }
+
+    def test_run_compare(
+        self, connection, models, instrument_inspector_conv, disable_pre_cache
+    ):
+        autogen_context, uo = models
+
+        autogenerate._produce_net_changes(autogen_context, uo)
+
+        sql_calls = len(instrument_inspector_conv.mock_calls)
+
+        eq_(
+            sql_calls,
+            self.expected[connection.dialect.name][
+                "pre_cache" if not disable_pre_cache else "no_pre_cache"
+            ],
+        )
